@@ -7,8 +7,6 @@ import sqlite3
 import time
 import unicodedata
 import warnings
-from copy import copy
-from pathlib import Path
 from typing import (
     Any,
     Callable,
@@ -24,7 +22,6 @@ from typing import (
 )
 
 import numpy as np
-from numpy import VisibleDeprecationWarning
 
 import qcodes as qc
 from qcodes.dataset.descriptions.dependencies import InterDependencies_
@@ -40,7 +37,6 @@ from qcodes.dataset.sqlite.connection import (
     atomic_transaction,
     transaction,
 )
-from qcodes.dataset.sqlite.database import connect
 from qcodes.dataset.sqlite.query_helpers import (
     VALUES,
     insert_column,
@@ -55,6 +51,7 @@ from qcodes.dataset.sqlite.query_helpers import (
     update_where,
 )
 from qcodes.utils.deprecate import deprecate
+from qcodes.utils.numpy_utils import list_of_data_to_maybe_ragged_nd_array
 
 log = logging.getLogger(__name__)
 
@@ -64,11 +61,23 @@ _unicode_categories = ('Lu', 'Ll', 'Lt', 'Lm', 'Lo', 'Nd', 'Pc', 'Pd', 'Zs')
 
 # in the current version, these are the standard columns of the "runs" table
 # Everything else is metadata
-RUNS_TABLE_COLUMNS = ["run_id", "exp_id", "name", "result_table_name",
-                      "result_counter", "run_timestamp", "completed_timestamp",
-                      "is_completed", "parameters", "guid",
-                      "run_description", "snapshot", "parent_datasets",
-                      "captured_run_id", "captured_counter"]
+RUNS_TABLE_COLUMNS = [
+    "run_id",
+    "exp_id",
+    "name",
+    "result_table_name",
+    "result_counter",
+    "run_timestamp",
+    "completed_timestamp",
+    "is_completed",
+    "parameters",
+    "guid",
+    "run_description",
+    "snapshot",
+    "parent_datasets",
+    "captured_run_id",
+    "captured_counter",
+]
 
 
 def is_run_id_in_database(conn: ConnectionPlus,
@@ -295,32 +304,16 @@ def get_parameter_data_for_one_paramtree(
     res_t = map(list, zip(*data))
 
     for paramspec, column_data in zip(paramspecs, res_t):
-        try:
-            if paramspec.type == "numeric":
-                # there is no reliable way to
-                # tell the difference between a float and and int loaded
-                # from sqlite numeric columns so always fall back to float
-                dtype: Optional[type] = np.float64
-            else:
-                dtype = None
-            with warnings.catch_warnings():
-                warnings.filterwarnings(
-                    "ignore",
-                    category=VisibleDeprecationWarning,
-                    message="Creating an ndarray from ragged nested sequences"
-                )
-                # numpy warns here and coming versions
-                # will eventually raise
-                # for ragged arrays if you don't explicitly set
-                # dtype=object
-                # It is time consuming to detect ragged arrays here
-                # and it is expected to be a relatively rare situation
-                # so fallback to object if the regular dtype fail
-                param_data[paramspec.name] = np.array(column_data, dtype=dtype)
-        except:
-            # Not clear which error to catch here. This will only be clarified
-            # once numpy actually starts to raise here.
-            param_data[paramspec.name] = np.array(column_data, dtype=object)
+        if paramspec.type == "numeric":
+            # there is no reliable way to
+            # tell the difference between a float and and int loaded
+            # from sqlite numeric columns so always fall back to float
+            dtype: Optional[type] = np.float64
+        else:
+            dtype = None
+        param_data[paramspec.name] = list_of_data_to_maybe_ragged_nd_array(
+            column_data, dtype
+        )
     return param_data, n_rows
 
 
@@ -790,19 +783,22 @@ def new_experiment(conn: ConnectionPlus,
 
     start_time = start_time or time.time()
     values = (name, sample_name, format_string, 0, start_time, end_time)
-
     curr = atomic_transaction(conn, query, *values)
     return curr.lastrowid
 
 
 # TODO(WilliamHPNielsen): we should remove the redundant
 # is_completed
-def mark_run_complete(conn: ConnectionPlus, run_id: int) -> None:
-    """ Mark run complete
+def mark_run_complete(
+    conn: ConnectionPlus, run_id: int, timestamp: Optional[float] = None
+) -> None:
+    """Mark run complete
 
     Args:
         conn: database connection
         run_id: id of the run to mark complete
+        timestamp: time stamp for completion. If None the function will
+            automatically get the current time.
     """
     query = """
     UPDATE
@@ -812,7 +808,9 @@ def mark_run_complete(conn: ConnectionPlus, run_id: int) -> None:
         is_completed=?
     WHERE run_id=?;
     """
-    atomic_transaction(conn, query, time.time(), True, run_id)
+    if timestamp is None:
+        timestamp = time.time()
+    atomic_transaction(conn, query, timestamp, True, run_id)
 
 
 def completed(conn: ConnectionPlus, run_id: int) -> bool:
@@ -859,15 +857,15 @@ def get_guid_from_run_id(conn: ConnectionPlus, run_id: int) -> str:
     return select_one_where(conn, "runs", "guid", "run_id", run_id)
 
 
-def get_guids_from_multiple_run_ids(db_path: Union[str, Path],
-                                    run_ids: Iterable[int]
-                                    ) -> List[str]:
+def get_guids_from_multiple_run_ids(
+    conn: ConnectionPlus, run_ids: Iterable[int]
+) -> List[str]:
     """
-    Retrieve guids of runs in the given database specified by their run ids.
+    Retrieve guids of runs in the connected database specified by their run ids.
     run ids are run_id in the database and not captured_run_id.
 
     Args:
-        db_path: The path to the database file.
+        conn: database connection
         run_ids: An integer iterable of run ids to get their guids.
 
     Returns:
@@ -876,19 +874,12 @@ def get_guids_from_multiple_run_ids(db_path: Union[str, Path],
 
     guids: List[str] = []
 
-    try:
-        conn = connect(db_path)
-        for run_id in run_ids:
-            if run_exists(conn=conn, run_id=run_id):
-                run_id_guid = get_guid_from_run_id(conn=conn,
-                                                   run_id=run_id)
-                guids.append(run_id_guid)
-            else:
-                raise RuntimeError(f'run id {run_id} does not'
-                                   ' exist in the database')
-    finally:
-        conn.close()
-        del conn
+    for run_id in run_ids:
+        if run_exists(conn=conn, run_id=run_id):
+            run_id_guid = get_guid_from_run_id(conn=conn, run_id=run_id)
+            guids.append(run_id_guid)
+        else:
+            raise RuntimeError(f"run id {run_id} does not exist in the database")
 
     return guids
 
@@ -1154,7 +1145,7 @@ def _insert_run(conn: ConnectionPlus, exp_id: int, name: str,
                 captured_counter = existing_captured_counter + 1
             else:
                 captured_counter = run_counter
-    formatted_name = format_table_name(format_string, name, exp_id,
+    formatted_name = format_table_name(format_string, 'results', exp_id,
                                        run_counter)
     table = "runs"
 
@@ -1405,10 +1396,18 @@ def update_parent_datasets(conn: ConnectionPlus,
         conn.cursor().execute(sql, (links_str, run_id))
 
 
-def set_run_timestamp(conn: ConnectionPlus, run_id: int) -> None:
+def set_run_timestamp(
+    conn: ConnectionPlus, run_id: int, timestamp: Optional[float] = None
+) -> None:
     """
     Set the run_timestamp for the run with the given run_id. If the
     run_timestamp has already been set, a RuntimeError is raised.
+
+    Args:
+        conn: database connection
+        run_id: id of the run to mark complete
+        timestamp: time stamp for completion. If None the function will
+            automatically get the current time.
     """
 
     query = """
@@ -1424,15 +1423,17 @@ def set_run_timestamp(conn: ConnectionPlus, run_id: int) -> None:
 
     with atomic(conn) as conn:
         c = conn.cursor()
-        timestamp = one(c.execute(query, (run_id,)), 'run_timestamp')
-        if timestamp is not None:
-            raise RuntimeError('Can not set run_timestamp; it has already '
-                               f'been set to: {timestamp}')
+        old_timestamp = one(c.execute(query, (run_id,)), "run_timestamp")
+        if old_timestamp is not None:
+            raise RuntimeError(
+                "Can not set run_timestamp; it has already "
+                f"been set to: {old_timestamp}"
+            )
         else:
-            current_time = time.time()
-            c.execute(cmd, (current_time, run_id))
-            log.info(f"Set the run_timestamp of run_id {run_id} to "
-                     f"{current_time}")
+            if timestamp is None:
+                timestamp = time.time()
+            c.execute(cmd, (timestamp, run_id))
+            log.info(f"Set the run_timestamp of run_id {run_id} to " f"{timestamp}")
 
 
 def add_parameter(conn: ConnectionPlus,
@@ -1711,11 +1712,30 @@ def get_metadata_from_run_id(
     return metadata
 
 
+def validate_meta_data(metadata: Mapping[str, Any]) -> None:
+    """
+    Validate metadata tags and values. Note that None is not a valid
+    metadata value, and keys should be valid SQLite column names
+    (i.e. contain only alphanumeric characters and underscores).
+
+    Args:
+        metadata: the metadata mapping (tags to values)
+    """
+    for tag, val in metadata.items():
+        if not tag.isidentifier():
+            raise KeyError(f'Tag {tag} is not a valid tag. '
+                            'Use only alphanumeric characters and underscores!')
+        if val is None:
+            raise ValueError(f'Tag {tag} has value None. '
+                              'That is not a valid metadata value!')
+
+
 def insert_meta_data(conn: ConnectionPlus, row_id: int, table_name: str,
                      metadata: Mapping[str, Any]) -> None:
     """
     Insert new metadata column and add values. Note that None is not a valid
-    metadata value
+    metadata value, and keys should be valid SQLite column names
+    (i.e. contain only alphanumeric characters and underscores).
 
     Args:
         - conn: the connection to the sqlite database
@@ -1723,10 +1743,7 @@ def insert_meta_data(conn: ConnectionPlus, row_id: int, table_name: str,
         - table_name: the table to add to, defaults to runs
         - metadata: the metadata to add
     """
-    for tag, val in metadata.items():
-        if val is None:
-            raise ValueError(f'Tag {tag} has value None. '
-                             ' That is not a valid metadata value!')
+    validate_meta_data(metadata)
     for key in metadata.keys():
         insert_column(conn, table_name, key)
     update_meta_data(conn, row_id, table_name, metadata)
@@ -1743,6 +1760,7 @@ def update_meta_data(conn: ConnectionPlus, row_id: int, table_name: str,
         - table_name: the table to add to, defaults to runs
         - metadata: the metadata to add
     """
+    validate_meta_data(metadata)
     update_where(conn, table_name, 'rowid', row_id, **metadata)
 
 
@@ -1752,7 +1770,9 @@ def add_meta_data(conn: ConnectionPlus,
                   table_name: str = "runs") -> None:
     """
     Add metadata data (updates if exists, create otherwise).
-    Note that None is not a valid metadata value.
+    Note that None is not a valid metadata value, and keys
+    should be valid SQLite column names (i.e. contain only
+    alphanumeric characters and underscores).
 
     Args:
         - conn: the connection to the sqlite database
